

from tendril.config import PLATFORM_CACHING_PROVIDER
from tendril.utils import log
logger = log.get_logger(__name__, log.DEFAULT)

from .providers import dummy
no_cache = dummy.cache

if PLATFORM_CACHING_PROVIDER == 'redis':
    logger.info("Using Redis for the platform level cache.")
    from .providers import redis
    platform_cache = redis.cache
else:
    logger.info("Platform level cache not configured.")
    platform_cache = dummy.cache





from tendril.utils.config import ConfigOption
from tendril.utils import log
logger = log.get_logger(__name__, log.DEFAULT)

depends = ['tendril.config.core']


config_elements_caching = [
    ConfigOption(
        'PLATFORM_CACHING_PROVIDER',
        "None",
        "Platform Caching Provider. "
        "Currently supports 'redis' and 'local'"
    ),
]


config_elements_redis = [
    ConfigOption(
        'REDIS_HOST',
        "'localhost'",
        "Redis Host"
    ),
    ConfigOption(
        'REDIS_PORT',
        "6379",
        "Redis Port"
    ),
    ConfigOption(
        'REDIS_DB',
        "0",
        "Redis database to use"
    ),
    ConfigOption(
        'REDIS_PASSWORD',
        "None",
        "Redis Password"
    ),
]


def load(manager):
    logger.debug("Loading {0}".format(__name__))
    manager.load_elements(config_elements_caching,
                          doc="Caching Configuration")
    if manager.PLATFORM_CACHING_PROVIDER == "redis":
        manager.load_elements(config_elements_redis,
                              doc="Redis Configuration")





import json

from tendril.config import PLATFORM_CACHING_PROVIDER

from tendril.utils import log
logger = log.get_logger(__name__, log.DEFAULT)


if PLATFORM_CACHING_PROVIDER == 'redis':
    logger.info("Using Redis for the platform level cache.")
    import redis
    from tendril.config import REDIS_HOST
    from tendril.config import REDIS_PORT
    from tendril.config import REDIS_DB
    from tendril.config import REDIS_PASSWORD
else:
    raise Exception("A valid PLATFORM_CACHING_PROVIDER is not configured "
                    "and a fallback is not presently implemented.")


redis_connection: redis.Redis = None


def _create_redis_connection():
    global redis_connection
    logger.info("Using Redis server at {}:{}:{} for transit caching"
                "".format(REDIS_HOST, REDIS_PORT, REDIS_DB))
    redis_connection = redis.Redis(host=REDIS_HOST,
                                   port=REDIS_PORT,
                                   db=REDIS_DB,
                                   password=REDIS_PASSWORD)


def _common(namespace=None, key=None):
    if not redis_connection:
        _create_redis_connection()

    if not key:
        raise NotImplementedError(
            "Redis transit caching requires a literal "
            "key to be provided."
        )

    if not namespace:
        logger.warn(f"No transit caching namespace was provided for "
                    f"key {key}. Using 'transit' instead.")

    cache_key = "{}:{}".format(namespace, key)
    return cache_key


def write(value=None, namespace=None, ttl=None, key=None, ser=json.dumps):
    cache_key = _common(namespace=namespace, key=key)
    logger.debug(f"Writing {value} to {cache_key}")
    redis_connection.set(cache_key, ser(value), ex=ttl)


def read(namespace=None, key=None, deser=json.loads):
    cache_key = _common(namespace=namespace, key=key)
    cached_value = redis_connection.get(cache_key)
    logger.debug(f"Read {cached_value} from {cache_key}")
    if cached_value:
        return deser(cached_value)
    else:
        return None


def delete(namespace=None, key=None):
    cache_key = _common(namespace=namespace, key=key)
    redis_connection.delete(cache_key)





from functools import wraps


def cache(namespace=None, ttl=None, key=None):
    def _wrapper_factory(func):
        @wraps(func)
        def wrapper_func(*args, **kwargs):
            return func(*args, **kwargs)
        return wrapper_func
    return _wrapper_factory





import json
import redis
from functools import wraps

from tendril.config import REDIS_HOST
from tendril.config import REDIS_PORT
from tendril.config import REDIS_DB
from tendril.config import REDIS_PASSWORD

from tendril.utils import log
logger = log.get_logger(__name__, log.DEFAULT)


redis_connection: redis.Redis = None


def _create_redis_connection():
    global redis_connection
    logger.info("Using Redis server at {}:{}:{} for caching"
                "".format(REDIS_HOST, REDIS_PORT, REDIS_DB))
    redis_connection = redis.Redis(host=REDIS_HOST,
                                   port=REDIS_PORT,
                                   db=REDIS_DB,
                                   password=REDIS_PASSWORD)


def cache(namespace=None, ttl=None, key=None, ser=json.dumps, deser=json.loads):
    if not redis_connection:
        _create_redis_connection()

    if not key:
        raise NotImplementedError(
            "Redis caching requires a key generator to be provided. "
            "The caching mechanism does not yet auto generate keys "
            "from functions arguments.")

    def _wrapper_factory(func):
        if not namespace:
            logger.warn("No caching namespace was provided for function {}. "
                        "Using 'generic_cache' instead.".format(func))

        @wraps(func)
        def wrapper_func(*args, **kwargs):
            cache_key = "{}:{}".format(namespace, key(*args, **kwargs))
            cached_value = redis_connection.get(cache_key)
            if cached_value:
                logger.debug("Cache Hit: {} {}".format(func, cache_key))
                return deser(cached_value)
            value = func(*args, **kwargs)
            logger.debug("Cache Miss: {} {}".format(func, cache_key))
            redis_connection.set(cache_key, ser(value), ex=ttl)
            return value

        return wrapper_func

    return _wrapper_factory
